""" Addon Information """
# ------------
# EventScripts Script-Addon:    BackRaw's USP Deathmatch (BUD, bud.py, 17.3kb)
# Author:                       BackRaw - http://forums.eventscripts.com/profile.php?mode=viewprofile&u=13683
#                                         http://addons.eventscripts.com/addons/user/13683
# Version:                      1.0
# ------------

""" Imports """
# ------------
# EventScripts Core-Module                - ES-functionality
import es
# Gamethread                              - Delayed functions
from gamethread import delayed
# Random                                  - Randomized choices
from random import choice
# Source Python Extensions                - Spawning players, safely removing weapons and entities
from spe.games.shared import removeEntityByIndex
from spe.games.cstrike import respawn
# ------------

""" SpawnPoints Class """
# ------------
# This small class will simply hold spawn-points, created by player-deaths.
# ------------
class SpawnPoints(object):
    def __init__(self):
        self.splist = []
    def getRandomSpawnPoint(self):
        return choice(self.splist)
sp = SpawnPoints()

""" USPDM Class """
# ------------
# This class will handle events, giving USP and entities and so on...
# ------------
class USPDM(object):
    """ When initialized, tell the Class to use the given UserID the whole time until it doesn't exist any more. """
    def __init__(self, userid):
        self.userid = userid
    # es_fire wrapper for easier using
    def fire(self, cmd):
        es.server.queuecmd('es_xfire %s %s' % (self.userid, cmd))
    # es_give wrapper for easier using
    def give(self, cmd):
        es.server.queuecmd('es_xgive %s %s' % (self.userid, cmd))
    # function wrapper, de-/activates buyzone and such functions
    def funcs(self, arg):
        arg = arg.capitalize()
        self.fire('func_buyzone %s' % arg)
        self.fire('func_bomb_target %s' % arg)
        self.fire('func_hostage_rescue %s' % arg)
    # strip-players function
    def strip(self):
        self.fire('player_weaponstrip Kill')
        es.server.queuecmd('es_xentcreate %s player_weaponstrip' % self.userid)
        self.fire('player_weaponstrip Strip')
    # player_spawn
    def spawn(self):
        self.funcs('disable')
        self.strip()
        self.give('item_assaultsuit')
        delayed(0, self.give, 'weapon_usp')
        if len(sp.splist):
            self.setSpawnLocation()
    def setSpawnLocation(self):
        es.server.queuecmd('es_xsetpos %s %s' % (self.userid, str(sp.getRandomSpawnPoint()).replace('(', '').replace(')', '')))
        self.checkStuck()
    def checkStuck(self):
        loc = es.getplayerlocation(self.userid)
        for x in filter(lambda x: str(x) != self.userid and es.getplayerlocation(x) == loc, es.getUseridList()):
            self.setSpawnLocation()
    # player_death
    def death(self):
        x, y, z = es.getplayerlocation(self.userid)
        if (x, y, z) not in sp.splist:
            sp.splist.append((x, y, z))
        respawn(int(self.userid))
        delayed(0, remove)

""" Call this Instance instead of calling the Class itself... """
def uspdm(userid):
    if es.exists('userid', userid):
        return USPDM(userid)
    raise TypeError, 'UserID %s does not exist!' % userid

""" Remove weapons wrapper """
def remove():
    entlist = es.createentitylist()
    for x in filter(lambda x: entlist[x]['classname'].startswith('weapon_') and es.getindexprop(x, 'CBaseEntity.m_hOwnerEntity') == -1, entlist):
        removeEntityByIndex(x)
    for x in es.getUseridList():
        uspdm(x).fire('cs_ragdoll Kill')

# ------------
# Strip the spawning player off his/her weapons and give him/her an USP.
# ------------
def player_spawn(ev):
    if int(ev['es_userteam']) > 1:
        uspdm(ev['userid']).spawn()

# ------------
# Give full ammo when the player reloads his/her USP.
# ------------
def weapon_reload(ev):
    uspdm(ev['userid']).give('info_null')

# ------------
# Give full ammo when the player fires his/her USP on empty clip.
# ------------
def weapon_fire_on_empty(ev):
    uspdm(ev['userid']).give('info_null')

# ------------
# Add the dead player's location to the spawn-points if it doesn't exist.
# Give the attacker full Armor, Health and Ammo.
# ------------
def player_death(ev):
    attacker = ev['attacker']
    uspdm(ev['userid']).death()
    pl = uspdm(attacker)
    pl.give('info_null')
    pl.give('item_assaultsuit')
    es.setplayerprop(attacker, 'CBasePlayer.m_iHealth', 100)

# ------------
# On round_end, delete every spawn-point created.
# ------------
def round_end(ev):
    sp.splist = []

""" EventScripts Main-Control functions """
# ------------
# On load, set the public variable and turn on noisy.
# ------------
def load():
    es.ServerVar('bud_ver', '1.0').makepublic()
    es.doblock('corelib/noisy_on')

# ------------
# On unload, delete every spawn-point created, turn off noisy, and enable buy-functions and so on for every player. 
# ------------
def unload():
    sp.splist = []
    es.doblock('corelib/noisy_off')
    for userid in es.getUseridList():
        uspdm(userid).funcs('enable')

""" End Code """
